%%    Favago Transport Programming Interface - CreateHamiltonians
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%  Returns an object providing interface to create and store Hamiltonian of the scattering region
%%
function Handles = CreateHamiltonians(Opt, param)

GeneralMessages          = Messages();
GeneralValidationHandles = ValidationFunctions();

kulso_szabfokok      = [];
Hscatter             = [];
fazis_mtx_scatter    = [];
coordinates          = [];

height               = [];
width                = [];
M                    = []; %number of sites in one slab

SegedfuggvenyekHandles = Segedfuggvenyek;

Handles.CreateScatterH = @CreateScatterH;
Handles.Write = @(MemberName, input)MemberWrite(MemberName, input);
Handles.Read  = @(MemberName)MemberRead(MemberName);
Handles.Clear = @(MemberName)MemberClear(MemberName);
Handles.saveScatter = @saveScatter;
Handles.loadScatter = @loadScatter;  
Handles.projectHamiltonian2Spin = @projectHamiltonian2Spin;
%Handles.CalcKulsoSzabfokok = @kulso_szabfokok_szamolas;    


 

%% CreateScatterH
% Creates a Hamiltonian of a rectangle shaped area on hexagonal lattice or from Hamiltonians given in Surface_tmp
function CreateScatterH( varargin )  
    p = inputParser;
    p.addParamValue('Surface_tmp', []);        
    p.parse(varargin{:});       
    Surface_tmp     = p.Results.Surface_tmp;
        
    shape = param.scatter.shape;
    
    % TODO!!!!!!!!!!
    width = shape.width;
    height = shape.height; % number of unit cells!!!!!!!!
    
    if isempty(Surface_tmp)
        Lead_End_Type = shape.Lead_End_Type;
        Atom_Type = shape.Atom_Type;       
    
        createH = Hex_Lead_Hamiltonians();
        if strcmp(Opt.Lattice_Type, 'H')
            [H0,H1,coordinates_unitcell] = createH.Graphene_Lead_Hamiltonians(param.scatter, width, Lead_End_Type, Atom_Type);
        elseif strcmp(Opt.Lattice_Type, 'Silicene')
            [H0,H1,coordinates_unitcell] = createH.Silicene_Lead_Hamiltonians(param.scatter, width, Lead_End_Type, Atom_Type);
        else
            [H0,H1,coordinates_unitcell] = createH.Graphene_Lead_Hamiltonians(param.scatter, width, Lead_End_Type, Atom_Type);
        end
    else
       H0 = Surface_tmp.Read( 'H0' );
       H1 = Surface_tmp.Read( 'H1' );
       coordinates_unitcell = Surface_tmp.Read( 'coordinates' ); 
    end
    
    H1adj = H1';
    M = size(H0,1);
    
    coordinates.x = zeros( M*height,1 );
    coordinates.y = zeros( M*height,1 );
    if isfield( coordinates_unitcell, 'spinup' )
        coordinates.spinup = false( M*height,1 );
    end
    Hscatter = sparse([],[],[],M*height, M*height);         
    
    for idx = 1:height
        Hscatter( (idx-1)*M+1 : idx*M, (idx-1)*M+1:idx*M ) = H0;
         if idx < height
       		Hscatter( idx*M+1:(idx+1)*M, (idx-1)*M+1:idx*M ) = H1adj;
       	end
        
       	if idx > 1
       		Hscatter( (idx-2)*M+1:(idx-1)*M, (idx-1)*M+1:idx*M ) = H1;
        end  
        
        coordinates.x( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.x + coordinates_unitcell.a(1)*idx;
        coordinates.y( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.y + coordinates_unitcell.a(2)*idx; 
        
        if isfield( coordinates, 'spinup' )
            coordinates.spinup( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.spinup;
        end
        
        if isfield( coordinates, 'BdG_u' )
            coordinates.BdG_u( (idx-1)*M+1 : idx*M ) = coordinates_unitcell.BdG_u;
        end
        
    end
    
    
end

%% projectHamiltonian2Spin
% projects Hamiltonian to spin states of $s=\pm1$
    function projectHamiltonian2Spin( s )
        if s == 1
            coordinates.x = coordinates.x(coordinates.spinup);
            coordinates.y = coordinates.y(coordinates.spinup);
            Hscatter = Hscatter( coordinates.spinup, coordinates.spinup);
        elseif s == -1
            coordinates.x = coordinates.x(~coordinates.spinup);
            coordinates.y = coordinates.y(~coordinates.spinup);
            Hscatter = Hscatter( ~coordinates.spinup, ~coordinates.spinup);          
        else
            disp('spin variable = +/-1')
        end
        
        coordinates = rmfield(coordinates, 'spinup');
        M = M/2;
        
        
    end

%% saveScatterHamiltonian
% saves the Hamiltonian of the scattering region
    function saveScatter( filename )
        if ~exist('filename', 'var')
            filename = 'Hscatter.mat';
        end
        save(filename,'Hscatter','coordinates', 'param', 'fazis_mtx_scatte', 'width', 'height', 'M')
    end


% ScatterHamiltonian_Square
% Negyszogracson szamolja a Hamiltonit --- Obsolete (rather create Hamiltonian from the Lead Hamiltonians)
function ScatterHamiltonian_Square()

shape = param.scatter.shape;
Nx        = size(shape, 1);
index_mtx = zeros(Nx,3);


idx = 1;
while idx <= Nx
    minimum          = min(shape(idx,1:2));
    maximum          = max(shape(idx,1:2));
    ny               = SegedfuggvenyekHandles.egesz_resz(maximum) - SegedfuggvenyekHandles.egesz_resz(minimum);
    index_mtx(idx,:) = [idx,ny,SegedfuggvenyekHandles.egesz_resz(minimum)];    
    idx = idx + 1;
end
Scatter_Geometry.eltolas = index_mtx(:,3);

size_Hscatter = sum(index_mtx(:,2));
Hscatter      = zeros(1,size_Hscatter);
szabfokok     = 0;

index_mtx = [[0 0 0];index_mtx;[0 0 0]];
idx = 2;

while idx <= Nx + 1

    ny   = index_mtx(idx,2);
    min1 = index_mtx(idx,3);
    ny2  = index_mtx(idx+1,2);
    min2 = index_mtx(idx+1,3);
    
    diffmin = min2-min1;
    diffmax = min2+ny2 - min1 - ny;
    
    ny_coupling = min([ny-diffmin,ny]); ny_coupling = min([ny_coupling+diffmax,ny_coupling]); 
    ny_coupling_matrixsize = max([ny_coupling,0]);
    h1          = sparse(1:ny_coupling_matrixsize, 1:ny_coupling_matrixsize, -param.scatter.vargamma, ny_coupling_matrixsize, ny_coupling_matrixsize);    
    
    if diffmin > 0
        h1             = [zeros(diffmin,ny_coupling_matrixsize);h1]; 
        sorok_szama    = ny_coupling+diffmin;
        oszlopok_szama = ny_coupling;
    elseif diffmin < 0
        h1             = [zeros(ny_coupling_matrixsize,-diffmin),h1];
        sorok_szama    = ny_coupling;
        oszlopok_szama = ny_coupling-diffmin;
    else
        sorok_szama    = ny_coupling;
        oszlopok_szama = ny_coupling;
    end
    
    
    if diffmax > 0
        h1             = [h1,zeros(sorok_szama,diffmax)];
        oszlopok_szama = oszlopok_szama+diffmax;
    elseif diffmax < 0
        h1           = [h1;zeros(-diffmax,oszlopok_szama)];
        sorok_szama  = sorok_szama-diffmax;
    end      
 
    if min(size(h1)) == 0
        h1 = zeros(ny,0);
    end
    h0           = sparse(1:ny,1:ny, param.scatter.epsilon, ny, ny) - sparse(1:ny-1,2:ny, param.scatter.vargamma, ny, ny) - sparse(2:ny,1:ny-1, param.scatter.vargamma, ny, ny);
    Hscatter_sor = [sparse([],[],0,ny,szabfokok),h0,2*h1,sparse([],[],0,ny,size_Hscatter-szabfokok-ny-oszlopok_szama)]; 
    szabfokok    = szabfokok + ny;
    Hscatter     = [Hscatter;Hscatter_sor]; 
    

idx = idx+1;
end

Hscatter(1,:) = [];
Hscatter      = (Hscatter' + Hscatter)/2;

index_mtx(1,:)     = [];
index_mtx(Nx+1,:)  = [];

if Opt.Potential == 1
    ApplyPotential(index_mtx);
end
index_mtx(:,3)     = [];
Scatter_Geometry.index_mtx = index_mtx;

saveScatterHamiltonian()

end 

% ApplyPotential
% Apply onsite potential on the sites. --- Osolete
function ApplyPotential(index_mtx)
    HandleForLoad = LoadFromFile('Scalar_Potential.mat');
    Hscatter = HandleForLoad.LoadVariable('V', 'NoEmpty', 'On');
    Idx_max = length(index_mtx(:,1));
    idx     = 1;
    szab_fokok = 0;
    while idx<= Idx_max
        Hscatter(szab_fokok+1:szab_fokok+index_mtx(idx,2), szab_fokok+1:szab_fokok+index_mtx(idx,2)) = ...
                                Hscatter(szab_fokok+1:szab_fokok+index_mtx(idx,2), szab_fokok+1:szab_fokok+index_mtx(idx,2)) + ...
                                sparse( 1:index_mtx(idx,2), 1:index_mtx(idx,2), ...
                                        V(index_mtx(idx,3)+1:index_mtx(idx,3)+index_mtx(idx,2),idx), ...
                                        index_mtx(idx,2), index_mtx(idx,2));
        szab_fokok = szab_fokok + index_mtx(idx,2);
        idx        = idx + 1;
    end       
        
        
end


%% LoadHamiltonian
% loads Scatter Hamiltonian
    function loadScatter( filename )
    if ~exist('filename', 'var')
        filename = 'Hscatter.mat';
    end    
    disp('Loading scatter Hamiltonian from file')
    HandleForLoad = LoadFromFile( filename );
    Hscatter = HandleForLoad.LoadVariable('Hscatter', 'NoEmpty', 'On');
    coordinates = HandleForLoad.LoadVariable('coordinates', 'NoEmpty', 'On');
    fazis_mtx_scatter = HandleForLoad.LoadVariable('fazis_mtx_scatte', 'NoEmpty', 'Off');
    param = HandleForLoad.LoadVariable('param', 'NoEmpty', 'On');
    M = HandleForLoad.LoadVariable('M', 'NoEmpty', 'On');
    width = HandleForLoad.LoadVariable('width', 'NoEmpty', 'Off');
    height = HandleForLoad.LoadVariable('haight', 'NoEmpty', 'On');
    HandleForLoad.ClearLoadedVariable();
end

%% MemberWrite
%   Sets the value of an element in the object.
    function MemberWrite(MemberName, input)
        
        if strcmp(MemberName, 'kulso_szabfokok')
            if isvector(input)
                kulso_szabfokok = input;
            else
                GeneralMessages.BadInputTypeNoDefault('kulso_szabfokok', 'vector')
            end
        elseif strcmp(MemberName, 'Hscatter')
            if ~isvector(input)
                Hscatter = input;
            else
                GeneralMessages.BadInputTypeNoDefault('Hscatter', 'matrix')
            end
        elseif strcmp(MemberName, 'fazis_mtx_scatter')
            if ~isvector(input)
                fazis_mtx_scatter = input;
            else
                GeneralMessages.BadInputTypeNoDefault('fazs_mtx_scatter', 'matrix')
            end  
        elseif strcmp(MemberName, 'coordinates')
            coordinates = input;
        elseif strcmp(MemberName, 'param')
            if ~GeneralValidationHandles.ValidatingFieldNames({'scatter', 'Leads', 'Coupling'}, 'param', input, 'Isstruct', 'On', 'FieldNumber', 'On', 'Ismember', 'On')
                param = input;
            else 
                GeneralMessages.BadInputTypeNoDefault('param', 'structure with proper fieldnames')
            end
        else
            disp(['No member to write with name ',MemberName]);
        end
    end

%% MemberRead
%   Reads the value of an element in the object.
    function ret = MemberRead(MemberName)
        if strcmp(MemberName, 'kulso_szabfokok')
            ret = kulso_szabfokok;
        elseif strcmp(MemberName, 'param')
            ret = param;
        elseif strcmp(MemberName, 'Hscatter')
            ret = Hscatter;
         elseif strcmp(MemberName, 'height')
            ret = height;
         elseif strcmp(MemberName, 'width')
            ret = width;
         elseif strcmp(MemberName, 'M')
            ret = M;   
         elseif strcmp(MemberName, 'coordinates')
            ret = coordinates;           
         elseif strcmp(MemberName, 'fazis_mtx_scatter')
            ret = fazis_mtx_scatter;          
        else
            disp(['No member to read with name ',MemberName]);
        end
    end

%% MemberClear
%   Clears the value of an element in the object.
    function MemberClear(MemberName)
        if strcmp(MemberName, 'kulso_szabfokok')
            kulso_szabfokok = [];
        elseif strcmp(MemberName, 'Hscatter')
            Hscatter = [];
         elseif strcmp(MemberName, 'coordinates')
            coordinates = [];           
        elseif strcmp(MemberName, 'fazis_mtx_scatter')
            fazis_mtx_scatter = [];          
        else
            disp(['No member to clear with name ',MemberName]);
        end
        
    end


% End of nested functions

end % Global end

