<?php $object_name = Ribbon; ?>
<h1 class="methods" class="methods"><?php echo $object_name; ?></h1>

<?php include ('licence.html'); ?>
  <p class="caption">Description</p>
  <p class="syntax">Returns an interface to perform transport calculations on a finite ribbon.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface = Ribbon(varargin)</p>

<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">width</p>Integer. Gives the number of the atomic sites in the cross section of the ribbon.</li>
      <li><p class="strucfield">height</p>Integer. Gives the height of the ribbon in units of the lattice vector.</li>
      <li><p class="strucfield">filenameIn</p>Input filename for the xml input structure.</li>
      <li><p class="strucfield">filenameOut</p>Output filename for the xml input structure.</li>
      <li><p class="strucfield">E</p>The energy used in the calculations.</li>
      <li><p class="strucfield">EF</p>The Fermi energy in eV.</li>
      <li><p class="strucfield">phi</p>A vector of pair potential phases in the left (1st) and right (2nd) superconductor</li>
      <li><p class="strucfield">silent</p>Set true for suppress the displaying of the output messages.</li>
      <li><p class="strucfield">PNpotential</p>The height of the pn potential across the ribbon.</li>
      <li><p class="strucfield">PNeta</p>The width of the pn potential step in units of the total width of the ribbon. ($$0 less PNeta less 1$$)</li>
  </ul>
</td>
</tr>
</table>

<p class="caption">Attributes</p>
<ul>
	<li><?php $method_name = "WS" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p> An instance of class handle <a href="index.php?menu=structures&location=ws_Ribbon" >ws_Ribbon</a>.</li>
	<li><?php $method_name = "Opt" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the structure <a href="index.php?menu=structures&location=Structures#Opt" >Opt</a></li>
	<li><?php $method_name = "param" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the structure <a href="index.php?menu=structures&location=Structures#param" >param</a></li>
	<li><?php $method_name = "G" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The Greens function of the scattering region.</li>
</ul>


<p class="caption">Methods</p>
<ul>
	<li><?php $method_name = "Transport" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CreateScatter" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "ShiftLeads" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "setEnergy" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "setsFermiEnergy" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CustomDysonFunc" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CalcFiniteGreensFunction" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CreateRibbon" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "ApplyMagneticField" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "DecimateInterface" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "setHandlesForMagneticField" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "AddPNPotential" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "createPNpotential" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "DecimationFunction" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "GetWorkspace" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
</ul>

<?php $method_name = "Transport" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the conductance of a finite ribbon connected to the leads in the "contact/scattering center/contact" arrangement. Use for development purpose only.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> [Conductivity,aspect_ratio,Conductance,ny,DeltaC] = interface.<?php echo $method_name; ?>(Energy) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Energy
</td>
<td class="parameter_values">
The energy to be used in the calculations.
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
Conductivity
</td>
<td class="parameter_values">
The calculated conductivity ($$Conductivity = Conductance/aspect_ratio$$).
</td>
</tr>
<tr>
<td class="parameter_names">
aspect_ratio
</td>
<td class="parameter_values">
The aspect_ratio of the scattering region ($$aspect_ratio = width/height$$).
</td>
</tr>
<tr>
<td class="parameter_names">
Conductance
</td>
<td class="parameter_values">
The calculated conductance in the units of $$2h/e$$.
</td>
</tr>
<tr>
<td class="parameter_names">
ny
</td>
<td class="parameter_values">
The number of the open channels in the leads.
</td>
</tr>
<tr>
<td class="parameter_names">
DeltaC
</td>
<td class="parameter_values">
The standard deviation of the calculated transmission probabilities.
</td>
</tr>
</table>
  <br></br><br></br><br></br>




<?php $method_name = "CreateScatter" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Creates an instance of the interface <a href="index.php?location=CreateHamiltoninans&menu=interfaces">CreateHamiltoninans</a> for the scattering region.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>() </p>

  <br></br><br></br><br></br>





<?php $method_name = "ShiftLeads" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Shifts the on-site potential in the leads.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( Energy ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Energy
</td>
<td class="parameter_values">
The energy to shift the on-site potential in the leads.
</td>
</tr>
</table>
  <br></br><br></br><br></br>





<?php $method_name = "setEnergy" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Sets the energy to be used in the calculations.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( Energy ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Energy
</td>
<td class="parameter_values">
The energy to be used in the calculations.
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<?php $method_name = "CustomDysonFunc" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Custom Dyson function for connecting the leads to the scattering region in the "contact/scattering center/contact" arrangement.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> Gret = interface.<?php echo $method_name; ?>( varargin ) </p>



<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">gfininv</p>The inverse of the Greens function of the scattering region. For default the inverse of the attribute <a href="#G">G</a> is used.</li>
	  <li><p class="strucfield">constant_channels</p>Logical value. Set true (default) to keep constatnt the number of the open channels for each energy value, or false otherwise.</li>
  </ul>
</td>
</tr>
</table>
</br>
<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
Gret
</td>
<td class="parameter_values">
The calculated Greens function.
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<?php $method_name = "CalcFiniteGreensFunction" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the surface Greens function of a finite ribbon. The calculated Greens function is stored in the <a href="#G">G</a> attribute.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( varargin ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">gauge_trans</p>Logical value. Set true to perform gauge transformation on the Green's function and on the Hamiltonians.</li>
  </ul>
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<?php $method_name = "CreateRibbon" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Creates an instance of the interface <a href="index.php?location=Surface_Green_function&menu=interfaces">Surface_Green_function</a> describing the unit cell in the ribbon. The created interface is stored in the attribute <a href="index.php?menu=structures&location=ws_Ribbon#Surface_tmp">ws_ribbon.Surface_tmp</a>. On hexagonal lattice a list <a href="index.php?menu=structures&location=ws_Ribbon#Surface_interface">ws_ribbon.Surface_interface</a> containing of instances of <a href="index.php?location=Surface_Green_function&menu=interfaces">Surface_Green_function</a> is also created. The members of the list are to be used as the interface regions between the leads and the scattering region.
<p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( vavargin ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">justHamiltonians</p>Logical value. Set true to create the Hamiltonian of the unit cell without performing any further calculations.</li>
  </ul>
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<?php $method_name = "ApplyMagneticField" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Applies the magnetic filed in the Hamiltonian of the unit cell.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>() </p>


  <br></br><br></br><br></br>





<?php $method_name = "DecimateInterface" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Decimates the Hamiltonians of the interface regions stored in <a href="index.php?menu=structures&location=ws_Ribbon#Surface_interface">ws_ribbon.Surface_interface</a> (needed in hexagonal lattice).</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( idx ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
idx
</td>
<td class="parameter_values">
The ordinal number of the instance of the interface <a href="index.php?location=Surface_Green_function&menu=interfaces">Surface_Green_function</a> in the list <a href="index.php?menu=structures&location=ws_Ribbon#Surface_interface">ws_ribbon.Surface_interface</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<?php $method_name = "setHandlesForMagneticField" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Sets the function handles of the vector potentials and gauge fields for applying the magnetic field in the Hamiltonians.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( varargin ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">scatter</p>Function handle of the vector potential to be used in the scattering region.</li>
      <li><p class="strucfield">lead</p>Function handle of the vector potential to be used in the leads.</li>
      <li><p class="strucfield">gauge_field</p>Function handle of the gauge transformation.</li>
  </ul>
</td>
</tr>
</table>
  <br></br><br></br><br></br>




<?php $method_name = "AddPNPotential" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Adds the p-n potential to the ribbon Hamiltonian.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( Surface_tmp ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Surface_tmp
</td>
<td class="parameter_values">
An instance of interface <a href="index.php?location=Surface_Green_function&menu=interfaces">Surface_Green_function</a>.
</td>
</tr>
</table>

</br>

  <br></br><br></br><br></br>





<?php $method_name = "createPNpotential" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Creates the p-n potential to be applied in the Hamiltonians.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> pot = interface.<?php echo $method_name; ?>() </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
pot
</td>
<td class="parameter_values">
The vector of the created p-n potential on each site in the cross section.
</td>
</tr>
</table>


  <br></br><br></br><br></br>



<?php $method_name = "DecimationFunction" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Performs the decimation procedure on the inverse Green's Function.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> ret = interface.<?php echo $method_name; ?>( kulso_szabfokok, ginv ) </p>

<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
kulso-szabfokok
</td>
<td class="parameter_values">
The vector of the sites to be kept after the decimation.
</td>
</tr>
<tr>
<td class="parameter_names">
ginv
</td>
<td class="parameter_values">
The matrix of the inverse Greens function to be decimated.
</td>
</tr>

</table>
</br>
<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
The matrix of the decimated inverse Greens function.
</td>
</tr>
</table>


  <br></br><br></br><br></br>


<?php $method_name = "GetWorkspace" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Return the handle class <a href="index.php?location=ws_Ribbon&menu=structures">ws_ribbon</a> of the workspace of the interface <a href="index.php?location=Ribbon&menu=interfaces">Ribbon</a>. Changes in fields of the workspace are not always safe, since the reinitialization of the interface is not performed.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>(  ) </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
An instance of handle class <a href="index.php?location=ws_Ribbon&menu=structures">ws_ribbon</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>


