<?php $object_name = SNSJosephson; ?>
<h1 class="methods" class="methods"><?php echo $object_name; ?></h1>

<?php include ('licence.html'); ?>
  <p class="caption">Description</p>
  <p class="syntax">Returns an interface to calculate the DC Josephson current in the SNS arrangement.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface = SNSJosephson( varargin )</p>

<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">width</p>Integer. Gives the number of the atomic sites in the cross section of the ribbon.</li>
      <li><p class="strucfield">height</p>Integer. Gives the height of the ribbon in units of the lattice vector.</li>
      <li><p class="strucfield">B</p>The strength of the magnetic field in the units of Tesla.</li>
      <li><p class="strucfield">T</p>The absolute temperature in Kelvins.</li>
      <li><p class="strucfield">filenameIn</p>Input filename for the xml input structure.</li>
      <li><p class="strucfield">filenameOut</p>Output filename for the xml input structure.</li>
      <li><p class="strucfield">E</p>The energy used in the calculations.</li>
      <li><p class="strucfield">EF</p>The Fermi energy in eV.</li>
      <li><p class="strucfield">phi</p>A vector of pair potential phases in the left (1st) and right (2nd) superconductor</li>
      <li><p class="strucfield">silent</p>Set true for suppress the displaying of the output messages.</li>
  </ul>
</td>
</tr>
</table>



<p class="caption">Attributes</p>
<ul>
	<li><?php $method_name = "WS" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p> An instance of class handle <a href="index.php?location=ws_SNSJosephson&menu=structures" >ws_SNSJosephson</a>.</li>
	<li><?php $method_name = "k_B" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The Boltzmann constant.</li>
	<li><?php $method_name = "beta" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>$$1/(k_B*T)$$</li>
	<li><?php $method_name = "T" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p> The absolute temperature in Kelvins.</li>
	<li><?php $method_name = "ribbon" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p> An instance of the interface <a href="index.php?location=Ribbon&menu=interfaces" >Ribbon</a>.</li>
	<li><?php $method_name = "WS_ribbon" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of class handle <a href="index.php?location=ws_Ribbon&menu=interfaces" >ws_Ribbon</a>.</li>
	<li><?php $method_name = "gfininv" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The inverse of the Greens function of the scattering region.</li>
	<li><?php $method_name = "BandWidth" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the structure <a href="index.php?menu=structures&location=Structures#BandWidth" >BandWidth</a>.</li>
</ul>

<p class="caption">Exported interfaces</p>
<ul>
	<li><a href="index.php?location=Ribbon&menu=interfaces" >ribbon</a></li>
</ul>


<p class="caption">Methods</p>
<ul>
	<li><?php $method_name = "Transport" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "create_scatter_GreensFunction" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CreateHandlesForMagneticField" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CurrentCalc_continuum" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CurrentCalc_discrete" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "getBandWidth" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "GetWorkspace" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
</ul>

<?php $method_name = "Transport" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the transmission probability of the BdG quasiparticles in the SNS arrangement.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> [Conductance,ny,DeltaC] = interface.<?php echo $method_name; ?>(Energy, B) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Energy
</td>
<td class="parameter_values">
The energy to be used in the calculations.
</td>
</tr>
<tr>
<td class="parameter_names">
B
</td>
<td class="parameter_values">
The strength of the magnetic field in the units of Tesla.
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
Conductance
</td>
<td class="parameter_values">
The calculated conductance in the units of $$2h/e$$.
</td>
</tr>
<tr>
<td class="parameter_names">
ny
</td>
<td class="parameter_values">
The number of the open channels in the leads.
</td>
</tr>
<tr>
<td class="parameter_names">
DeltaC
</td>
<td class="parameter_values">
The standard deviation of the calculated transmission probabilities.
</td>
</tr>
</table>
  <br></br><br></br><br></br>





<?php $method_name = "CurrentCalc_continuum" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the Josephson current induced by the continuous scattering states.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> [currentvec, current_surf] = interface.<?php echo $method_name; ?>( DeltaPhi_vec ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
DeltaPhi_vec
</td>
<td class="parameter_values">
The vector of the superconducting phase differences.
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
currentvec
</td>
<td class="parameter_values">
The vector of the integrated Josephson current in the energy range of the scattering states for each phase difference given in the parameter DeltaPhi_vec. 
</td>
</tr>
<tr>
<td class="parameter_names">
current_surf
</td>
<td class="parameter_values">
A matrix containing the calculated Josephson current for each phase difference (rows) and energy point (columns).
</td>
</tr>
</table>
  <br></br><br></br><br></br>





<?php $method_name = "CurrentCalc_discrete" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the Josephson current induced by the continuous scattering states.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> currentvec = interface.<?php echo $method_name; ?>( DeltaPhi_vec, varargin ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
DeltaPhi_vec
</td>
<td class="parameter_values">
The vector of the superconducting phase differencies.
</td>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">range</p>String. Set 'ABS' (default) for the Andreev bound states, or 'NBS' for the normal bound states. </li>
  </ul>
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
currentvec
</td>
<td class="parameter_values">
The vector of the Josephson current summed over the discrete bound states for each phase difference given in the parameter DeltaPhi_vec. 
</td>
</tr>
</table>
  <br></br><br></br><br></br>







<?php $method_name = "CreateHandlesForMagneticField" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Creates function handles of the vector potentials and apply the magnetic filed in the ribbon Hamiltonians.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface.<?php echo $method_name; ?>() </p>


  <br></br><br></br><br></br>





<?php $method_name = "getBandWidth" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Determines the band width in the leads and in the scattering region. The results are stored in the attribute <a href="#BandWidth">BandWidth</a>.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface.<?php echo $method_name; ?>() </p>


  <br></br><br></br><br></br>




<?php $method_name = "create_scatter_GreensFunction" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the surface Greens function of the antidot. Sets the attribute <a href="#gfininv">gfininv</a> to the calculated results.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface.<?php echo $method_name; ?>( varargin ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">onlyAtEnds</p>Logical value. Set true to calculate the Greens function only at the ends of the scattering region, or false (default) to calculate the Greens function in several inner slabs as well.</li>
      <li><p class="strucfield">gauge_trans</p>Logical value. Set true to perform gauge transformation on the Green's function and on the Hamiltonians, or false (default) otherwise.</li>
  </ul>
</td>
</tr>
</table>


  <br></br><br></br><br></br>





<?php $method_name = "GetWorkspace" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Return a class handle <a href="index.php?location=ws_SNSJosephson&menu=structures">ws_SNSJosephson</a> of the workspace of the interface <a href="index.php?location=SNSJosephson&menu=interfaces">SNSJosephson</a>. Changes in fields of the workspace are not always safe, since the reinitialization of the interface is not performed.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>(  ) </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
An instance of class handle <a href="index.php?location=ws_antidot&menu=structures">ws_antidot</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>


