%    Minimal conductivity of graphene - based on Favago TPI v4.4
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
function MinimalConductivity( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

toCalculate = 1;
Continue    = 0;
outfilename = ['Ribbon_Conductivity',num2str( filenum ),'.mat'];
outputdir   = [];

height = 170;
Energy = 0.001; %In EV


ribbon_hamiltonian    = [];



Widthmin = 10;
Widthdelta = 20;
Widthnum = 30;%100;%200;
Widthvec = Widthmin:Widthdelta:Widthmin+Widthnum*Widthdelta;

% calculated conductance (and number of open channels) as a function of width/height
Conductivity = NaN(Widthnum+1,1); 
nyitott_csatornak = NaN(Widthnum+1,1);
DeltaC      = NaN(Widthnum+1,1);
aspectRatio = NaN(Widthnum+1,1); 

if Continue && toCalculate
  HandleForLoad = LoadFromFile( outfilename);
  Conductivity = HandleForLoad.LoadVariable('Conductivity', 'NoEmpty', 'On');
  aspectRatio = HandleForLoad.LoadVariable('aspectRatio', 'NoEmpty', 'On');
  DeltaC = HandleForLoad.LoadVariable('DeltaC', 'NoEmpty', 'Off');
  nyitott_csatornak = HandleForLoad.LoadVariable('nyitott_csatornak', 'NoEmpty', 'On');
  Widthvec = HandleForLoad.LoadVariable('Widthvec', 'NoEmpty', 'On');
  Energy = HandleForLoad.LoadVariable('Energy', 'NoEmpty', 'Off');
  HandleForLoad.ClearLoadedVariable(); 
end


CalculateTranspor();
Egeszabra();




%% Calculates transport
    function CalculateTranspor()
        
    
        if ~toCalculate 
             disp('Loading Calculated data')
            HandleForLoad = LoadFromFile( outfilename);
            Conductivity = HandleForLoad.LoadVariable('Conductivity', 'NoEmpty', 'On');
            aspectRatio = HandleForLoad.LoadVariable('aspectRatio', 'NoEmpty', 'On');
            DeltaC = HandleForLoad.LoadVariable('DeltaC', 'NoEmpty', 'On');
            Widthvec = HandleForLoad.LoadVariable('Widthvec', 'NoEmpty', 'On');
            Energy = HandleForLoad.LoadVariable('Energy', 'NoEmpty', 'Off');
            HandleForLoad.ClearLoadedVariable();
            
            setOutputDir();
            return
        end
        
        setOutputDir();
        
        
         if Continue && toCalculate
           indexes2calc = find( Conductivity == 0 );
        else
           indexes2calc = 1:length(Widthvec);
        end
        
        for idx = 1:length(indexes2calc)       
            width = Widthvec(indexes2calc(idx));
            ribbon_hamiltonian = Ribbon('width', width, 'height', height );     
        
            disp(' ----------------------------------')
            disp('Plotting conductivity:')
            Egeszabra();        
        
        
            disp(' ')
            disp(' ')
            disp(' ----------------------------------')
            disp('Calculating Conductance for the given aspect ratio')           
        

            
            tic
            [Conductivity_tmp,aspectRatio_tmp,~,nyitott_csatornak_num,DeltaC_tmp] = ribbon_hamiltonian.Transport( Energy );
            toc
            
            Conductivity(indexes2calc(idx)) = Conductivity_tmp;
            nyitott_csatornak(indexes2calc(idx)) = nyitott_csatornak_num(1);
            DeltaC(indexes2calc(idx)) = DeltaC_tmp;
            aspectRatio(indexes2calc(idx)) = aspectRatio_tmp;
            
            
            
            save( [outputdir,'/',outfilename], 'Conductivity', 'aspectRatio', 'nyitott_csatornak', 'Widthvec', 'Energy', 'DeltaC', 'height' );
            disp([ num2str(indexes2calc(idx)/length(Widthvec)*100),' % calculated of the conductance.'])
            disp( ' ' )
            
            
            
        end
                
    end



%% sets the output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = resultsdir;        
    end




    function Egeszabra( )
        

        
        figure1 = figure('XVisual','0x21 (TrueColor, depth 24, RGB mask 0xff0000 0xff00 0x00ff)','Renderer','painters',...
    'Colormap',[1 1 0.994791686534882;0.954365074634552 0.954365074634552 0.932043671607971;0.908730149269104 0.908730149269104 0.869295656681061;0.863095223903656 0.863095223903656 0.80654764175415;0.817460358142853 0.817460358142853 0.74379962682724;0.771825432777405 0.771825432777405 0.68105161190033;0.726190507411957 0.726190507411957 0.618303596973419;0.680555582046509 0.680555582046509 0.555555582046509;0.643518567085266 0.629629671573639 0.518518567085266;0.606481492519379 0.578703701496124 0.481481492519379;0.569444477558136 0.527777791023254 0.444444477558136;0.532407402992249 0.476851880550385 0.407407432794571;0.495370388031006 0.425925940275192 0.370370388031006;0.458333343267441 0.375 0.333333343267441;0.421296298503876 0.32407408952713 0.296296298503876;0.384259253740311 0.273148149251938 0.259259253740311;0.347222208976746 0.222222223877907 0.222222223877907;0.33983451128006 0.217494085431099 0.217494085431099;0.332446783781052 0.212765961885452 0.212765961885452;0.325059086084366 0.208037823438644 0.208037823438644;0.31767138838768 0.203309699892998 0.203309699892998;0.310283690690994 0.19858156144619 0.19858156144619;0.302895963191986 0.193853422999382 0.193853422999382;0.2955082654953 0.189125299453735 0.189125299453735;0.288120567798615 0.184397161006927 0.184397161006927;0.280732840299606 0.179669037461281 0.179669037461281;0.273345142602921 0.174940899014473 0.174940899014473;0.265957444906235 0.170212760567665 0.170212760567665;0.258569717407227 0.165484637022018 0.165484637022018;0.251182019710541 0.160756498575211 0.160756498575211;0.243794322013855 0.156028375029564 0.156028375029564;0.236406609416008 0.151300236582756 0.151300236582756;0.229018896818161 0.146572098135948 0.146572098135948;0.221631199121475 0.141843974590302 0.141843974590302;0.214243486523628 0.137115836143494 0.137115836143494;0.206855788826942 0.132387712597847 0.132387712597847;0.199468076229095 0.127659574151039 0.127659574151039;0.192080363631248 0.122931443154812 0.122931443154812;0.184692665934563 0.118203312158585 0.118203312158585;0.177304953336716 0.113475181162357 0.113475181162357;0.16991725564003 0.108747042715549 0.108747042715549;0.162529543042183 0.104018911719322 0.104018911719322;0.155141845345497 0.0992907807230949 0.0992907807230949;0.14775413274765 0.0945626497268677 0.0945626497268677;0.140366420149803 0.0898345187306404 0.0898345187306404;0.132978722453117 0.0851063802838326 0.0851063802838326;0.12559100985527 0.0803782492876053 0.0803782492876053;0.118203304708004 0.075650118291378 0.075650118291378;0.110815599560738 0.0709219872951508 0.0709219872951508;0.103427894413471 0.0661938562989235 0.0661938562989235;0.0960401818156242 0.0614657215774059 0.0614657215774059;0.0886524766683578 0.0567375905811787 0.0567375905811787;0.0812647715210915 0.0520094558596611 0.0520094558596611;0.0738770663738251 0.0472813248634338 0.0472813248634338;0.0664893612265587 0.0425531901419163 0.0425531901419163;0.059101652354002 0.037825059145689 0.037825059145689;0.0517139472067356 0.0330969281494617 0.0330969281494617;0.0443262383341789 0.0283687952905893 0.0283687952905893;0.0369385331869125 0.0236406624317169 0.0236406624317169;0.029550826177001 0.0189125295728445 0.0189125295728445;0.0221631191670895 0.0141843976452947 0.0141843976452947;0.0147754130885005 0.00945626478642225 0.00945626478642225;0.00738770654425025 0.00472813239321113 0.00472813239321113;0 0 0]);

        fontsize = 9;
        
        %% Conductivity    
        
        %x_lim = [min(coordinates.x(:,1)) max(coordinates.x(:,1)) ];
        %x_lim = [0 rightside];
        
        theoretical_limit = 2/pi; %Theoretical minimal conductivity
        
        %y_lim = [0 2];
        
        indexek = logical( (abs(DeltaC) <= 0.5) & (aspectRatio~=0) & ~isnan(DeltaC) );
        x_lim = [0 max(aspectRatio(indexek))];
        
        if norm(x_lim) == 0
            x_lim = [0 1];
        end
        
        Position = [0.08 0.58 0.53 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
        numerics = plot(aspectRatio(indexek), Conductivity(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);       
        %plot(Widthvec(indexek), nyitott_csatornak(indexek), 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);
        try
            plot(aspectRatio(indexek), DeltaC(indexek), 'Linewidth', 1, 'color', [0 1 0], 'Parent', axes_cond);
        catch
            disp('No DeltaC data present')
        end
         plot( [0 max(aspectRatio(indexek))], ones(1,2)*theoretical_limit, 'r',  'Parent', axes_cond);
        
               
        
        
        %plot(Evec, nyitott_csatornak, 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);    
        
         % Create xlabel
        %xlabel_position = [0 0 0];
        xlabel('W/L','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %xlabel_handle = get(axes_cond,'XLabel');  
        %set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

        % Create ylabel
        %ylabel_position = [0 0 0];
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %ylabel_handle = get(axes_cond,'YLabel'); 
        %set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);
        
        
        %set(get(get(numerics,'Annotation'),'LegendInformation'), 'IconDisplayStyle','off'); % Exclude line from legend

        fig_legend = legend(axes_cond, {'conductivity', 'unitarity'});%, 'FontSize', fontsize, 'FontName','Times New Roman')
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')

        
        print('-depsc2', [outputdir,'/Minimal_Conductivity_', num2str(filenum),'.eps'])
        print('-dpdf', [outputdir,'/Minimal_Conductivity_', num2str(filenum),'.pdf'])
        if toCalculate
            close(figure1)
        end
        
        
    end




end
