%%    Favago Transport Programming Interface - OverlapingIntegrals
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    Provides interface for applying the overlap integrals.
%%
function handles = OverlapIntegrals()

S_Hscatter  = [];
S_Hhopping  = [];
S_H0        = [];
S_H1        = [];



handles.ApplyOverlapInScatter  = @ApplyOverlapInScatter;
handles.ApplyOverlapInLeads    = @ApplyOverlapInLeads;


%% ApplyOverlapInScatter
% Applies overlap integrals in the scattering region
    function ApplyOverlapInScatter( Way2Hamiltonian, E )
        
        Hamiltoni       = Way2Hamiltonian.Read( 'Hscatter' );
        Way2Hamiltonian.Clear( 'Hscatter' );
        LoadOverlap( 'Hscatter' );
        
        if ~all(size(Hamiltoni) == size(S_Hscatter))
            warning('MATLAB:WrongOverlapData', 'Wrong Overlap data for Hscatter. No Overlap considered.');
        else
            Hsize = size(Hamiltoni,1);
            Hamiltoni = Hamiltoni - (S_Hscatter - sparse(1:Hsize, 1:Hsize, 1, Hsize, Hsize))*E;
        end
        
        S_Hscatter = [];
        Way2Hamiltonian.Write( 'Hscatter', Hamiltoni );
    end

        


 %% ApplyOverlapInLeads
 % Applies overlap integrals in the leads
    function ApplyOverlapInLeads( Surface_tmp, E )
        
        H0        = Surface_tmp.Read( 'H0' );
        H1        = Surface_tmp.Read( 'H1' );
        Surface_tmp.Clear( 'H0' );
        Surface_tmp.Clear( 'H1' );
        
        Hanyadik_Lead = Surface_tmp.Read( 'Hanyadik_Lead' );
        LoadOverlap( 'Lead' , 'Hanyadik_Lead', Hanyadik_Lead);
        
        if ~all(size(H0) == size(S_H0)) || ~all(size(H1) == size(S_H1))
            warning('MATLAB:WrongOverlapData', 'Wrong Overlap data for H0 or H1. No Overlap considered.');
        else
            Hsize = size(H0,1);
            H0  = H0 - (S_H0 - sparse(1:Hsize, 1:Hsize, 1, Hsize, Hsize))*E;
            H1  = H1 - S_H1*E;
        end
      
        Surface_tmp.Write( 'H0', H0 );
        Surface_tmp.Write( 'H1', H1 );
    end


% Loading Overlap matrices
    function LoadOverlap(varname, varargin)
        
        p = inputParser;
        p.addParamValue('Hanyadik_Lead', 0);
        p.parse(varargin{:});

        Hanyadik_Lead  = p.Results.Hanyadik_Lead;
        
        if strcmp(varname, 'Hscatter')
            HandleForLoadOverlap  = LoadFromFile('S_Hscatter.mat');
            S_Hscatter            = HandleForLoadOverlap.LoadVariable('S_Hscatter');
        elseif strcmp(varname, 'Coupling')
            HandleForLoadOverlap  = LoadFromFile('S_Coupling.mat');
            S_Hhopping            = HandleForLoadOverlap.LoadVariable('S_Hhopping');
        elseif strcmp(varname, 'Lead')
            HandleForLoadOverlap  = LoadFromFile(['S_Lead_',num2str(Hanyadik_Lead),'.mat']);
            S_H0                  = HandleForLoadOverlap.LoadVariable('S_H0');
            S_H1                  = HandleForLoadOverlap.LoadVariable('S_H1');
        end
        HandleForLoadOverlap.ClearLoadedVariable();
    end
end % Global end
