%%    Favago Transport Programming Interface - Surface_Green_function
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%  Returns an object providing interface to calculate the Greend functions
%%
function ret = Surface_Green_function(Opt, param, varargin)


usingDualModes    = []; %if True the dual modes are used to calculate Greens function, if false, left and rigth-side modes are used.
Lead_Orientation  = [];
Hanyadik_Lead     = [];
Way2Hamiltonian   = [];
GinfSzamolo       = [];
Lead_End_Type     = [];
Atom_Type         = [];
M             = [];
params        = [];  %Lead parameters
retarted      = [];  % retarted or advanced Greensfunction
inputparser();

kulso_szabfokok = [];
coordinates   = [];  %geometry data of the sites in the unit cell and a 1D lattice vector (possible fields: x, y, for the xy coordinates, "a" for the lattice vector, "spin" contains logical values for spin up, BdG_u contains logical values for electron like BdG components
H0            = [];
H1            = [];
H1adj         = [];
H00           = [];
fazis_mtx_H0  = [];
fazis_mtx_H1  = [];
modusmtx      = [];
modusmtx_left = [];
k             = [];
csoportseb    = [];

k_p           = [];
k_m           = [];
vcsop_p       = [];
vcsop_m       = [];
modusmtx_p    = [];
modusmtx_p_left = [];
d_modusmtx_m  = [];
modusmtx_m    = [];
modusmtx_m_left = [];
d_modusmtx_p  = [];
Normamtx      = [];
invNormamtx   = [];
gsurf         = [];
gsurfinv      = [];
ginf          = [];
gfin          = [];



ret.Write = @MemberWrite;
ret.Read  = @MemberRead;
ret.Clear = @MemberClear;

ret.CreateHamiltonians   = @CreateHamiltoniansForLead;
ret.TrukkosSajatertekek  = @TrickyEigenvalues;
ret.Group_Velocity       = @Group_Velocity;
ret.SurfaceGreenFunction = @SurfaceGreenFunction;
ret.InfGreenFunction     = @InfGreenFunction;
ret.FiniteGreenFunction  = @FiniteGreenFunction;
ret.SurfaceGreen_simple  = @SurfaceGreen_simple;
ret.CreateClone          = @CreateClone;
ret.SaveLeads            = @saveLeads;
ret.ShiftLead            = @ShiftLead;
ret.ShiftCoordinates     = @ShiftCoordinates;
ret.AddPotential         = @AddPotential;
ret.CalcSpektrum         = @CalcSpektrum;
ret.isSuperconducting    = @isSuperconducting;

ret.Reset                = @Reset;


%------------------------------------------------------------------
% Nested functions

%% InfGreenFunction
%
% Calculates the Greens function of the infinite lead between slabs z1 and z2. The calculated Greens function is stored within the current object. 
    function ret = InfGreenFunction(z1,z2, varargin)
        szetvalaszto();
        
        p = inputParser;
        if z1 >= z2
            p.addParamValue('z1points', 1:size(modusmtx_p,1));
            p.addParamValue('z2points', 1:size(modusmtx_p,2));
        else
            p.addParamValue('z1points', 1:size(modusmtx_m,1));
            p.addParamValue('z2points', 1:size(modusmtx_m,2));            
        end
        p.parse(varargin{:});

        z1points = p.Results.z1points;
        z2points = p.Results.z2points;
        
        ginf = GinfSzamolo(z1, z2, z1points, z2points);
        ret = ginf;
        
%        d_modusmtx_m = [];
%        d_modusmtx_p = [];
    end
        
    %--------------------------------------------------------------------------
    % Calculate Greens function with dual modes
    function ginf = GinfSzamoloWithDualModes(z1, z2, z1points, z2points) 
        NormamtxSzamolo();
        ginf = 0;
        
        if z1 >= z2
            for idx = 1:M
                xi_p    = modusmtx_p(z1points,idx);
                d_xi_p  = d_modusmtx_p(idx,:);
                ginf = ginf + xi_p*d_xi_p*exp(1i*k_p(idx)*Lead_Orientation*(z1-z2)); 
            end
        else
            for idx = 1:M          
                xi_m    = modusmtx_m(z1points,idx);
                d_xi_m  = d_modusmtx_m(idx,:);
                ginf = ginf + xi_m*d_xi_m*exp(1i*k_m(idx)*Lead_Orientation*(z1-z2));  
            end
        end
        
        ginf = ginf*invNormamtx(:,z2points);
    end
    %--------------------------------------------------------------------------
    
        %--------------------------------------------------------------------------
    % Calculate Greens function with left modes
    function ginf = GinfSzamoloWithLeftModes(z1, z2, z1points, z2points) 
        ginf = 0;
        
        if z1 >= z2
            for idx = 1:M
                xi_p       = modusmtx_p(z1points,idx);
                xi_p_left  = modusmtx_p_left(idx,z2points);
                ginf = ginf + xi_p*xi_p_left*exp(1i*k_p(idx)*Lead_Orientation*(z1-z2)) / (1i*vcsop_p(idx)); 
            end
        else
            for idx = 1:M          
                xi_m      = modusmtx_m(z1points,idx);
                xi_m_left = modusmtx_m_left(idx,z2points);
                ginf = ginf + xi_m*xi_m_left*exp(1i*k_m(idx)*Lead_Orientation*(z1-z2)) / (-1i*vcsop_m(idx));  
            end
        end        
    end
    %--------------------------------------------------------------------------

%% FiniteGreenFunction
%   Calculates the Greens function of a finite piece of the lead between slabs z1 and z2. The calculated Greens function is stored by the current object.
    function ret = FiniteGreenFunction(z1,z2) 
        
        szetvalaszto();
        NormamtxSzamolo();        
        
        gfin = zeros(4*M,4*M);
        
        if z1 >= z2
            points = 1:size(modusmtx_p,1);
            dpoints = 1:size(modusmtx_p,1);
        else
            points = 1:size(modusmtx_m,1);
            dpoints = 1:size(modusmtx_m,1);  
        end
        
        gtmp = GinfSzamolo(z1-1, z1-1, points, dpoints);
        gfin(1:M,1:M)              = gtmp;
        gfin(1+M:2*M, 1+M:2*M)     = gtmp;
        gfin(1+2*M:3*M, 1+2*M:3*M) = gtmp;              
        gfin(1+3*M:4*M, 1+3*M:4*M) = gtmp;
        
        gtmp = GinfSzamolo(z1-1, z1, points, dpoints);
        gfin(1:M,1+M:2*M) = gtmp;
        gfin(1+2*M:3*M,1+3*M:4*M) = gtmp;
        
        gtmp = GinfSzamolo(z1, z1-1, points, dpoints);
        gfin(1+M:2*M,1:M) = gtmp;
        gfin(1+3*M:4*M, 1+2*M:3*M) = gtmp;    
        
        gtmp = GinfSzamolo(z1-1, z2, points, dpoints);
        gfin(1:M,1+2*M:3*M) = gtmp; 
        
        gtmp = GinfSzamolo(z2, z1-1, points, dpoints);
        gfin(1+2*M:3*M,1:M) = gtmp; 
                
        gtmp = GinfSzamolo(z1-1, z2+1, points, dpoints);
        gfin(1:M,1+3*M:4*M) = gtmp; 
                
        gtmp = GinfSzamolo(z2+1, z1-1, points, dpoints);
        gfin(1+3*M:4*M,1:M) = gtmp;   
        
        gtmp = GinfSzamolo(z1, z2, points, dpoints);
        gfin(1+M:2*M,1+2*M:3*M) = gtmp; 
        
        gtmp = GinfSzamolo(z2, z1, points, dpoints);
        gfin(1+2*M:3*M,1+M:2*M) = gtmp; 
        
        gtmp = GinfSzamolo(z1, z2+1, points, dpoints);
        gfin(1+M:2*M,1+3*M:4*M) = gtmp; 
        
        gtmp = GinfSzamolo(z2+1, z1, points, dpoints);
        gfin(1+3*M:4*M,1+M:2*M) = gtmp;   
        
        gfin = inv(gfin);
        gfin = inv(gfin( 1+M:3*M, 1+M:3*M ));        
        ret = gfin;
        
    end

%% SurfaceGreenFunction
%   Calculates the surface Greens function of a semi-infinite lead. The calculated Greens function is stored by the current object.
    function SurfaceGreenFunction()        
            
        szetvalaszto();
        NormamtxSzamolo(); 
        if usingDualModes
            GsurfSzamoloWithDualModes();
        else
            GsurfSzamoloWithLeftModes();
        end        
        
        
        
        %--------------------------------------------------------------------------
        function GsurfSzamoloWithDualModes()  
        
            gsurf_m = 0;
            gsurf_p = 0;
            idx     = 1;
            while idx <= M
                xi_m    = modusmtx_m(:,idx); 
                d_xi_m  = d_modusmtx_m(idx,:);
                gsurf_m = gsurf_m + xi_m*d_xi_m*exp(-1i*k_m(idx)*Lead_Orientation); 
                xi_p    = modusmtx_p(:,idx); 
                d_xi_p  = d_modusmtx_p(idx,:);
                gsurf_p = gsurf_p + xi_p*d_xi_p*exp(1i*k_p(idx)*Lead_Orientation); 
                idx   = idx +1;       
            end

            %gsurf = (eye(M) - gsurf_p*gsurf_m)*inv(Normamtx);
            gsurf = (eye(M) - gsurf_m*gsurf_p)*invNormamtx;
            gsurfinv = inv(gsurf);%gsurf\eye(size(gsurf));%inv(gsurf);
        end
        %-------------------------------------------------------------------------- 
        
        
        %--------------------------------------------------------------------------
        function GsurfSzamoloWithLeftModes()  
            gsurf = zeros(2*M,2*M);
        
            points = 1:M;
            dpoints = 1:M;
            
            
            gtmp = GinfSzamolo(0, 0, points, dpoints);
            gsurf(1:M,1:M)              = gtmp;
            gsurf(1+M:2*M, 1+M:2*M)     = gtmp;
        
            gtmp = GinfSzamolo(-1, 0, points, dpoints);
            gsurf(1:M,1+M:2*M) = gtmp;
        
            gtmp = GinfSzamolo(0, -1, points, dpoints);
            gsurf(1+M:2*M,1:M) = gtmp;  
            
            gsurfinv = inv(gsurf);
            gsurfinv = gsurfinv( 1:M, 1:M );
            gsurf = inv(gsurfinv);
        
        
        end
        %--------------------------------------------------------------------------         
    end
        
    %--------------------------------------------------------------------------
    % Normalasi matrix
    function NormamtxSzamolo()
            
            if ~isempty(Normamtx) && ~isempty(invNormamtx)
               return 
            elseif usingDualModes
                NormamtxSzamoloWithDualModes();
            else
                NormamtxSzamoloWithLeftModes();
            end
                        
            
        %--------------------------------------------------------------
        function NormamtxSzamoloWithDualModes()
            
            calcDualModes();  
            Normamtx = 0;
            invNormamtx = 0;
            idx_m = 1;
            idx_p = 1;

            while idx_m <= M && idx_p <= M
                xi_m     = modusmtx_m(:,idx_m);
                d_xi_m   = d_modusmtx_m(idx_m,:);
                xi_p     = modusmtx_p(:,idx_p);
                d_xi_p   = d_modusmtx_p(idx_p,:);
        
                Normamtx = Normamtx +(xi_p*d_xi_p*exp(-1i*k_p(idx_p)) - xi_m*d_xi_m*exp(-1i*k_m(idx_m)))*Lead_Orientation; %normalasi matrix
                idx_p    = idx_p +1;    
                idx_m    = idx_m + 1;
            end
            
            Normamtx = H1'*Normamtx;
            invNormamtx = Normamtx\(eye(size(Normamtx)));%inv(Normamtx);
        end
        %---------------------------------------------------------------
        
        %--------------------------------------------------------------
        function NormamtxSzamoloWithLeftModes()
            
            invNormamtx = InfGreenFunction(0,0);
            Normamtx    = invNormamtx\(eye(size(invNormamtx)));
           
            
        end
        %---------------------------------------------------------------
    end
    %--------------------------------------------------------------------------    
    % Calculate dual modes       
    function  calcDualModes()
       if isempty(d_modusmtx_m) 
           display('Calculating dual modes for m')
           d_modusmtx_m = CalcDualModesEigenvec( k_m, modusmtx_m ); %a dualis modusok
       end
       
       if isempty(d_modusmtx_p) 
           display('Calculating dual modes for p')
           d_modusmtx_p = CalcDualModesEigenvec( k_p, modusmtx_p ); %a dualis modusok
       end

%	display('modusmtx inverse')       
%       norm( modusmtx_m*d_modusmtx_m - eye(size(modusmtx_m)) )
%       norm( modusmtx_p*d_modusmtx_p - eye(size(modusmtx_p)) )        

         % ---------------------------------------------------------------
         function d_modes = CalcDualModesEigenvec( k, modusmtx )        
            d_modes = (modusmtx)\eye(size(modusmtx)); % a dualis modusok            
        end
        
    end        
    %--------------------------------------------------------------------------
    % Szetvalasztja a jobbra es balra ahaladokat        
    function szetvalaszto()
        
        if isempty(csoportseb)
            return
        end
            
            tolerance  = 1e-6;
            modusmtx_p = zeros(M,M);
            modusmtx_p_left = zeros(M,M);
            k_p        = zeros(1,M);
            vcsop_p    = zeros(1,M);
            modusmtx_m = zeros(M,M);
            modusmtx_m_left = zeros(M,M);
            k_m        = zeros(1,M);
            vcsop_m    = zeros(1,M);
            
            if retarted
                indexes = logical( (real(csoportseb)*Lead_Orientation > 0 & abs(imag(k)) < tolerance) | imag(k)*Lead_Orientation > tolerance );
            else
                indexes = logical( (real(csoportseb)*Lead_Orientation < 0 & abs(imag(k)) < tolerance) | imag(k)*Lead_Orientation > tolerance );
            end
            k_p             = k(indexes);
            vcsop_p         = csoportseb(indexes);
            modusmtx_p      = modusmtx(:,indexes);
            modusmtx_p_left = modusmtx_left(indexes,:);
            
            k_m             = k(~indexes);
            vcsop_m         = csoportseb(~indexes);
            modusmtx_m      = modusmtx(:,~indexes);
            modusmtx_m_left = modusmtx_left(~indexes,:);
            
            modusmtx   = [];
            csoportseb = [];
            k          = [];
            
            %[k_p, modusmtx_p, vcsop_p] = sorbarendezo(k_p, modusmtx_p, vcsop_p);
            %[k_m, modusmtx_m, vcsop_m] = sorbarendezo(k_m, modusmtx_m, vcsop_m);
            %vcsop_p = abs(real(vcsop_p)) + imag(vcsop_p)*1i;
            %vcsop_m = abs(real(vcsop_m)) + imag(vcsop_m)*1i; 
        
        
        
        
    end
    %------------------------------------------------------------------
    

%% Group_Velocit 
%   Calculates the group velocities corresponding to the propagating states. The calculated group velocities are stored within the object.
    function Group_Velocity(varargin)
        
        p = inputParser;
        p.addParamValue('WithOverlap', []);
        p.parse(varargin{:});  
        
        WithOverlap    = p.Results.WithOverlap;
        
        csoportseb = zeros(1,2*M);
        d_modusmtx = modusmtx';
        
         
        if ~isempty(WithOverlap)
            ModusNormalas();            
            d_modusmtx = modusmtx';
        end
        
        GroupVelocityWithNoOverlap();
        
        modusmtx       = modusmtx*diag(exp(-1i*k/2).*sqrt(csoportseb));
        modusmtx_left  = diag(exp(-1i*k/2).*sqrt(csoportseb))*modusmtx_left/1i;
        RemormModusMtx()
        TestOfGroupVelocity()
        
        
        %-----------------------------------------------------------------
        function RemormModusMtx()
            for idx = 1:size(modusmtx,2)
                modusmtx_norm = norm(modusmtx(:,idx));
                modusmtx(:,idx) = modusmtx(:,idx)/modusmtx_norm;
                modusmtx_left(idx,:) = modusmtx_left(idx,:)*modusmtx_norm;
            end            
        end
        %----------------------------------------------------------------
        
        %-----------------------------------------------------------------
        function ModusNormalas()
            [S_H0, S_H1] = WithOverlap.LoadAndClear();
            
            idx = 1;
            while idx <= 2*M
            modusmtx(:,idx) = modusmtx(:, idx) /  sqrt(d_modusmtx(idx, :) * (S_H0 + S_H1*exp(1i*k(idx)) + S_H1'*exp(-1i*k(idx))) * modusmtx(:, idx));
                idx = idx + 1; 
            end           
            
        end
        
        
        %------------------------------------------------------------------
        function GroupVelocityWithNoOverlap()
        
            idx = 1;
            while idx <= 2*M
                csoportseb(idx) = 1i*d_modusmtx(idx,:)*(H1*exp(1i*k(idx))-H1'*exp(-1i*k(idx)))*modusmtx(:,idx)/(d_modusmtx(idx,:)*modusmtx(:,idx));
                idx = idx+1;
            end
        end
        %------------------------------------------------------------------
        
        %------------------------------------------------------------------
        % Tests whether equation (A4) in PRB 78, 035407 hold on
        function TestOfGroupVelocity()
            
            csoportseb_test = zeros(1,2*M);
        
            idx = 1;
            while idx <= 2*M
                csoportseb_test(idx) = 1i*modusmtx_left(idx,:)*(H1*exp(1i*k(idx))-H1'*exp(-1i*k(idx)))*modusmtx(:,idx);
                idx = idx+1;
            end
            
            difference = csoportseb_test - csoportseb;
            if norm(difference) > 1e-10
                disp( ['Test of group velocities failed: ', num2str(norm(difference))])
            end            
        end
        %------------------------------------------------------------------

    end
    %------------------------------------------------------------------



%% TrickyEigenvalues
%   Calculates the wave numbers corresponding to the propagating states at given energy. The calculated wave numbers are stored by within the object.
    function TrickyEigenvalues(E)
        
        if imag(E) < 0
            retarted = false;
        else
            retarted = true;
        end
        
        [H_eff,B_eff] = Effective_H(H0,H1,H1adj,M,E);
        [modusmtx,k]   = eig(H_eff,B_eff);
        [modusmtx_left,k_left]   = eig(H_eff.', B_eff.');
        
        clear H_eff;
        modusmtx_left = transpose(modusmtx_left);
        
        k              = diag(k);
        k_left         = diag(k_left); 
        
        pairLeft2Right();
        
        k = transpose(log(k)/1i);
        k_left = transpose(log(k_left)/1i);
        
        
        normfactors = diag(1./diag(modusmtx_left * B_eff * modusmtx ));
        modusmtx_left = normfactors*modusmtx_left;
        modusmtx       = modusmtx(1:M,:); 
        modusmtx_left  = modusmtx_left(:,1:M);        
                
        
        Normamtx       = [];
        invNormamtx    = [];
        modusmtx_p     = [];
        modusmtx_m     = [];
        d_modusmtx_p     = [];
        d_modusmtx_m     = [];
        
        
        %------------------------------------------------------------------
        % pair the left and right side eigenvectors
        function pairLeft2Right()
            k_left_unsorted = k_left;
            modusmtx_left_unsorted = modusmtx_left;
            
            for idx = 1:length(k)
               k_tmp = k(idx);
               [minval,jdx] = min( abs(k_left_unsorted-k_tmp) );
               k_left(idx)  = k_left_unsorted( jdx );               
               modusmtx_left(idx,:) = modusmtx_left_unsorted(jdx,:);
                
            end         
            
        end
        %------------------------------------------------------------------
        
        %------------------------------------------------------------------
        function [H_eff,B_eff] = Effective_H(H0,H1,H1adj,M,E)
            E_matrix = eye(M)*E;
            %H1inv = H1\eye(size(H1));%inv(H1);
            H_eff    = full([-(H0 - E_matrix),full(-H1adj);eye(M),zeros(M,M)]);  
            B_eff    = full([ H1, zeros(M,M); zeros(M,M), eye(M,M)]);
        end
        %------------------------------------------------------------------
    end

%% CreateHamiltoniansForLead
%   Creates the Hamiltonians H_0 and H_1 of the lead. The created Hamiltonians are stored by within the object. 
    function CreateHamiltoniansForLead( varargin )
        
        p = inputParser;
        p.addParamValue('toSave', 1);
        p.parse(varargin{:});

        toSave  = p.Results.toSave;
        
        setM();
        
        if Opt.Hleads == 1
            HandleForLoad = LoadFromFile(['Hamiltoni_Lead_',num2str(Hanyadik_Lead),'.mat']);
            H0            = HandleForLoad.LoadVariable('H0', 'NoEmpty', 'On');
            H1            = HandleForLoad.LoadVariable('H1', 'NoEmpty', 'On');
            H1adj         = H1';
            kulso_szabfokok = HandleForLoad.LoadVariable('kulso_szabfokok');
            Lead_Orientation = HandleForLoad.LoadVariable('Lead_Orientation');
            params        = HandleForLoad.LoadVariable('params');
            fazis_mtx_H0  = HandleForLoad.LoadVariable('fazis_mtx_H0', 'NoEmpty', 'Off');
            fazis_mtx_H1  = HandleForLoad.LoadVariable('fazis_mtx_H1', 'NoEmpty', 'Off');
            HandleForLoad.ClearLoadedVariable();            
        else
            if strcmp(Opt.Lattice_Type, 'S')
                [H0,H1,coordinates] = HamiltoniansSquare(); 
                H1adj = H1';
                kulso_szabfokok = (1:2*M);
            elseif strcmp(Opt.Lattice_Type, 'H') || strcmp(Opt.Lattice_Type, 'Graphene')
                createH = Hex_Lead_Hamiltonians();
                [H0,H1,coordinates] = createH.Graphene_Lead_Hamiltonians(params, M, Lead_End_Type, Atom_Type);
                H1adj = H1';
                kulso_szabfokok = [1:M, size(H0,1)+1:size(H0,1)+M];
            elseif strcmp(Opt.Lattice_Type, 'Silicene')
                createH = Hex_Lead_Hamiltonians();
                [H0,H1,coordinates] = createH.Silicene_Lead_Hamiltonians(params, M, Lead_End_Type, Atom_Type);
                H1adj = H1';
                kulso_szabfokok = [1:M, size(H0,1)+1:size(H0,1)+M];                
            end
            
            if Opt.BdG
                fnames = fieldnames( coordinates );
                for idx = 1:length(fnames)
                    fname = fnames{idx};
                    if strcmp(fname, 'a')
                        continue
                    end                    
                    coordinates.(fname) = [ coordinates.(fname); coordinates.(fname) ];
                end
                
                coordinates.BdG_u = [ true(size(H0,1),1); false(size(H0,1),1)];
                
                length_kulso_szabfokok = length(kulso_szabfokok)/2;
                kulso_szabfokok = [kulso_szabfokok(1:length_kulso_szabfokok), kulso_szabfokok(1:length_kulso_szabfokok)+size(H0,1), ...
                    kulso_szabfokok(length_kulso_szabfokok+1:end) + size(H0,1), kulso_szabfokok(length_kulso_szabfokok+1:end) + 2*size(H0,1)];
                pair_potential = params.pair_potential;
                H0 = [H0, eye(size(H0))*pair_potential; eye(size(H0))*conj(pair_potential), -conj(H0)];
                H1 = [H1, zeros(size(H1)); zeros(size(H1')), -conj(H1)];
                H1adj = H1';
                M = 2*M;
            end
            
            if toSave
                saveLeads();
            end
        end
  
        
        
        %------------------------------------------------------------------
        function [H0,H1,coordinates] = HamiltoniansSquare()
            epsilon  = params.epsilon;
            vargamma = params.vargamma;
            
            H0 = sparse(1:M,1:M,epsilon,M,M) - sparse(1:M-1,2:M,vargamma,M,M) - sparse(2:M,1:M-1,vargamma,M,M);
            H1 = sparse(1:M,1:M,-vargamma,M,M);
            
			coordinates = struct('x', [], 'y', []);
			coordinates.x = transpose(1:M);
			coordinates.y = zeros(M,1);
            coordinates.a = [0; 1];
        end
        %------------------------------------------------------------------
        
        
    end
    
    
%% CalcSpektrum
%   Creates the band structure of the lead.
    function spectrum = CalcSpektrum( varargin )
        
        p = inputParser;
        p.addParamValue('toPlot', 1, @isscalar);  
        p.addParamValue('ka_min', -pi, @isscalar);  
        p.addParamValue('ka_max', pi, @isscalar);  
        p.addParamValue('ka_num', 300, @isscalar);
        p.addParamValue('center', 0.001);  
        p.addParamValue('db', min([10,size(H0,1)]), @isscalar); 
        
        p.parse(varargin{:});    
        toPlot     = p.Results.toPlot;
        ka_min     = p.Results.ka_min;
        ka_max     = p.Results.ka_max;
        ka_num     = p.Results.ka_num;
        center = p.Results.center;
        db             = p.Results.db;
        

    display('Calculating spectrum')   
    
    
    deltak = (ka_max-ka_min)/ka_num;

    ka = ka_min;
    H1loc = H1;
    offset = mean(diag(H0));
    H0loc = H0 - eye(size(H0))*offset;


    abra     = [0 0];
 
    while ka<= ka_max
     
        H        = secular_H(H0loc,H1loc,ka);     
        E        = eigs(H, db, center);%2.97*sqrt(9/2*eta_B));    
        abra     = [abra;[ones(length(E),1)*ka,E]];%*(WN-1)*sqrt(3)/6,E]];
    
        ka      = ka + deltak;
    
    end
 
    abra(1,:) = [];
    abra(:,2) = abra(:,2);%/homega;
    spectrum = [abra(:,1), real(abra(:,2) + offset)];
 
    display('Spectrum calculated')
    
    if ~toPlot
        return
    end
    
    figure1 = figure();
    fontsize = 9;
    
    x_lim = [min(spectrum(:,1)) max(spectrum(:,1))];
    y_lim = [min(spectrum(:,2));max(spectrum(:,2))];

    Position = [0.5 0.58 0.33 0.4];
    axes_spectrum = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,...
                'xlim', x_lim,...
                'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
    plot(spectrum(:,1), spectrum(:,2),'.', 'MarkerSize', 4, 'Parent', axes_spectrum )  
        
    % Create xlabel
    xlabel_position = [0 0 0];
    xlabel('ka','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_spectrum);
    xlabel_handle = get(axes_spectrum,'XLabel');  
    set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

    % Create ylabel
    ylabel_position = [0 0 0];
    ylabel('E [eV]','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_spectrum);
    ylabel_handle = get(axes_spectrum,'YLabel'); 
    set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);  

    
        % --------------------------------------
        % Hamiltonian for the secular equation
        function H = secular_H(H0,H1,k)
        
            H = H0 + H1*exp(1i*k) + H1'*exp(-1i*k);
        
        end

 
    end   
    
    %------------------------------------------------------------------
%% saveLeads
%   Save Lead Hamiltonians  into a file 'Hamiltoni_Lead_' + num2str(Hanyadik_Lead) + '.mat'.   
    function saveLeads()
        save(['Hamiltoni_Lead_',num2str(Hanyadik_Lead),'.mat'], 'H0', 'H1', 'kulso_szabfokok', 'Lead_Orientation', 'fazis_mtx_H0', 'fazis_mtx_H1', 'params');
    end    
    %------------------------------------------------------------------
    
    
%% ShiftCoordinates
%   Shifts the coordinates of the sites by an integer multiple of the lattice vector.
    function ShiftCoordinates( shift )       
        coordinates.x = coordinates.x + shift*coordinates.a(1);
        coordinates.y = coordinates.y + shift*coordinates.a(2);        
    end
    
    %------------------------------------------------------------------
%% SurfaceGreen_simple
%   Calculates the surface Greens function of a semi-infinite lead by an analytical formula valid for square lattice without magnetic field. The calculated Greens function is stored by the current object.
    function SurfaceGreen_simple(E)

        epsilon  = params.epsilon;
        vargamma = params.vargamma;
        
        % a hullamszamvektor
        k     = kvektor();
        vcsop = 2*vargamma*sin(k);

        idx   = 1;
        gsurf = 0;
        Normamtx     = 0;
        modusmtx = zeros(M,M);
        while idx<=M
            xi       = sqrt(2/(M+1))*sin(idx*pi/(M+1)*(1:M)');
            gsurf    = gsurf - xi*xi'*exp(1i*k(idx))/vargamma;
            Normamtx = Normamtx + 2*1i*vargamma*(xi*xi')*sin(k(idx)); %normalasi matrix
            modusmtx(:,idx) = xi;
            idx      = idx + 1;
        end
            
        modusmtx_p = modusmtx;
        modusmtx_m = modusmtx;
        modusmtx   = [];
        [k_p,modusmtx_p,vcsop_p] = sorbarendezo(k,modusmtx_p,vcsop);
        [k_m,modusmtx_m,vcsop_m] = sorbarendezo(k,modusmtx_m,vcsop);
            
        k = [];
        clear vcsop;
           
            
        %------------------------------------------------------------------
        function k = kvektor()
            idx = 1;
            k   = zeros(1,M);
            while idx<=M
                k(idx) = acos((epsilon-E)/(2*vargamma) - cos(idx*pi/(M+1)));
                idx    = idx + 1;
            end
        end
        %------------------------------------------------------------------

        
    end
    %------------------------------------------------------------------
        
    % Sorbarendezi a sajatertekeket es fazisnormalja a hullamfuggvenyeket
    %--------------------------------------------------------------------------
    function [k, modusmtx, csopseb] = sorbarendezo(k_regi, modusmtx_regi, csopseb_regi)

        Idx           = length(k_regi);
        idx_temp      = 1;
        k             = zeros(Idx,1);
        csopseb       = zeros(1,Idx);
        modusmtx      = zeros(size(modusmtx_regi));
    
        while Idx >= 1
            [~,idx]        = max(abs(k_regi));
            k(idx_temp)          = k_regi(idx);   
            k_regi(idx)          = [];
            %modusmtx(:,idx_temp) = modusmtx_regi(:,idx)*exp(-1i*angle(modusmtx_regi(1,idx)))/norm(modusmtx_regi(:,idx));
            modusmtx(:,idx_temp) = modusmtx_regi(:,idx);%/norm(modusmtx_regi(:,idx));
            modusmtx_regi(:,idx) = [];
            csopseb(idx_temp)    = csopseb_regi(idx);
            csopseb_regi(idx)    = [];
            idx_temp             = idx_temp +1; 
            Idx                  = Idx-1; 
        end
 
    end
    %--------------------------------------------------------------------------
    
%% ShiftLead
%   Shifts the on-site energies in the leads by a given energy.
    function ShiftLead( Energy )        
        H0 = H0 + sparse(1:size(H0,1), 1:size(H0,2),Energy,size(H0,1),size(H0,2)); 
        params.epsilon = params.epsilon + Energy;
        Normamtx = [];
    end



%% AddPotential
%   Adds on-site potential to the Hamiltonian H-0.
    function AddPotential( V )        

        if ( size(V,2) == 1 ) || ( size(V,1) == 1 )
            if length(V) == size(H0,1)
                H0 = H0 + sparse(1:size(H0,1), 1:size(H0,2),V,size(H0,1),size(H0,2));
            else
                disp(' Wrong dimension of potential: 1')
                return
            end
        elseif norm(size(V) - size(H0)) < 1e-6
            H0 = H0 + V;            
        else
            disp(' Wrong dimension of potential: 2')
            return
        end
        
        display(' Potential added to lead')
        Normamtx = [];
    end

%% isSuperconducting
%   Test, whether the lead is in the superconducting phase or not.
    function ret = isSuperconducting()
        
        if ~Opt.BdG
            ret = false;
            return;
        end
        
        if abs(params.pair_potential) >= 1e-10
            ret = true;
        else
            ret = false;
        end
        
        return
            
    
    end

%% CreateClone
%   Creates a clone of the present Surface_Green_function object.
    function Surface_tmp = CreateClone()
        
        Surface_tmp = Surface_Green_function(Opt, param,...
                                            'Way2Hamiltonian', Way2Hamiltonian, ...
                                            'Hanyadik_Lead', Hanyadik_Lead,...
                                            'Lead_Orientation', Lead_Orientation);
                                        
        Surface_tmp.Write( 'H0', H0); 
        Surface_tmp.Write( 'H1', H1);
        Surface_tmp.Write( 'H1adj', H1adj);
        Surface_tmp.Write( 'fazis_mtx_H0', fazis_mtx_H0); 
        Surface_tmp.Write( 'fazis_mtx_H1', fazis_mtx_H1);        
        Surface_tmp.Write( 'H00', H00);
        Surface_tmp.Write( 'kulso_szabfokok', kulso_szabfokok);
		Surface_tmp.Write( 'coordinates', coordinates);
        Surface_tmp.Write( 'gsurf', gsurf);
        Surface_tmp.Write( 'ginf', ginf);
        Surface_tmp.Write( 'gfin', gfin);
        Surface_tmp.Write( 'Normamtx', Normamtx);
        Surface_tmp.Write( 'vcsop_p', vcsop_p);
        Surface_tmp.Write( 'vcsop_m', vcsop_m);
        Surface_tmp.Write( 'modusmtx_p', modusmtx_p);
        Surface_tmp.Write( 'modusmtx_m', modusmtx_m);
        Surface_tmp.Write( 'k_p', k_p);
        Surface_tmp.Write( 'k_m', k_m);
        Surface_tmp.Write( 'params', params);
        
        
        
    end   

%% Reset
%   Resets all elements in the object.
    function Reset()
        usingDualModes    = []; %if True the dual modes are used to calculate Greens function, if false, left and rigth-side modes are used.
        Lead_Orientation  = [];
        Hanyadik_Lead     = [];
        Way2Hamiltonian   = [];
        GinfSzamolo       = [];
        Lead_End_Type     = [];
        Atom_Type         = [];
        M             = [];
        params        = [];  %Lead parameters
        inputparser();

        kulso_szabfokok = [];
        coordinates   = [];  %geometry data of the sites in the unit cell and a 1D lattice vector
        H0            = [];
        H1            = [];
        H1adj         = [];
        H00           = [];
        fazis_mtx_H0  = [];
        fazis_mtx_H1  = [];
        modusmtx      = [];
        modusmtx_left = [];
        k             = [];
        csoportseb    = [];

        k_p           = [];
        k_m           = [];
        vcsop_p       = [];
        vcsop_m       = [];
        modusmtx_p    = [];
        modusmtx_p_left = [];
        d_modusmtx_m  = [];
        modusmtx_m    = [];
        modusmtx_m_left = [];
        d_modusmtx_p  = [];
        Normamtx      = [];
        invNormamtx   = [];
        gsurf         = [];
        gsurfinv      = [];
        ginf          = [];
        gfin          = [];
        
        retarted      = [];

        
        
    end


%% MemberWrite
%   Sets the value of an element in the object.
    function MemberWrite(MemberName, input)
        
        if strcmp(MemberName, 'H0')
            H0 = input;
        elseif strcmp(MemberName, 'H1')
            H1 = input; 
        elseif strcmp(MemberName, 'H1adj')
            H1adj = input;    
        elseif strcmp(MemberName, 'H00')
            H00 = input;
        elseif strcmp(MemberName, 'fazis_mtx_H0')
            fazis_mtx_H0 = input; 
        elseif strcmp(MemberName, 'fazis_mtx_H1')            
            fazis_mtx_H1 = input;  
        elseif strcmp(MemberName, 'Hamiltonian2Dec')
            H00   = input(1:M,1:M);
            H1    = input(1:M, M+1:2*M);
            H0    = input(M+1:2*M, M+1:2*M);
            H1adj = input( M+1:2*M, 1:M);
        elseif strcmp(MemberName, 'kulso_szabfokok')
            kulso_szabfokok = input;
        elseif strcmp(MemberName, 'coordinates')
            coordinates = input;
        elseif strcmp(MemberName, 'gsurf')
            gsurf = input;
        elseif strcmp(MemberName, 'ginf')
            ginf = input;
        elseif strcmp(MemberName, 'gfin')
            gfin = input;            
        elseif strcmp(MemberName, 'Normamtx')
            Normamtx = input;
        elseif strcmp(MemberName, 'vcsop_p')
            vcsop_p = input;
        elseif strcmp(MemberName, 'vcsop_m')
            vcsop_m = input;
        elseif strcmp(MemberName, 'modusmtx_p')
            modusmtx_p = input;
        elseif strcmp(MemberName, 'modusmtx_m')
            modusmtx_m = input;
        elseif strcmp(MemberName, 'd_modusmtx_p')
            d_modusmtx_p = input;
        elseif strcmp(MemberName, 'd_modusmtx_m')
            d_modusmtx_m = input;            
        elseif strcmp(MemberName, 'k_p')
            k_p = input;
        elseif strcmp(MemberName, 'k_m')
            k_m = input;
        elseif strcmp(MemberName, 'epsilon')
            params.epsilon = input;
        elseif strcmp(MemberName, 'vargamma')
            params.vargamma = input;
        elseif strcmp(MemberName, 'params')
            params = input;
        elseif strcmp(MemberName, 'param')
            param = input;
            Reset()
        else
            disp(['No member to write with name ',MemberName]);
        end
    end
%% MemberRead
%   Reads the value of an element in the object.
    function ret = MemberRead(MemberName)
        if strcmp(MemberName, 'H0')
            ret = H0;
        elseif strcmp(MemberName, 'H1')
            ret = H1;
        elseif strcmp(MemberName, 'H1adj')
            ret = H1adj;
        elseif strcmp(MemberName, 'H00')
            ret = H00;
        elseif strcmp(MemberName, 'fazis_mtx_H0')
            ret = fazis_mtx_H0; 
        elseif strcmp(MemberName, 'fazis_mtx_H1')            
            ret = fazis_mtx_H1;
        elseif strcmp(MemberName, 'Hamiltonian2Dec')
            ret = [H0,H1;H1adj,H0];
        elseif strcmp(MemberName, 'kulso_szabfokok')
            ret = kulso_szabfokok;
        elseif strcmp(MemberName, 'coordinates')
            ret = coordinates;
        elseif strcmp(MemberName, 'gsurf')
            ret = gsurf;
        elseif strcmp(MemberName, 'gsurfinv')
            ret = gsurfinv;            
        elseif strcmp(MemberName, 'ginf')
            ret = ginf; 
         elseif strcmp(MemberName, 'gfin')
            ret = gfin;            
        elseif strcmp(MemberName, 'Normamtx')
            ret = Normamtx;
        elseif strcmp(MemberName, 'vcsop_p')
            ret = vcsop_p;
        elseif strcmp(MemberName, 'vcsop_m')
            ret = vcsop_m;
        elseif strcmp(MemberName, 'modusmtx_p')
            ret = modusmtx_p;
        elseif strcmp(MemberName, 'modusmtx_m')
            ret = modusmtx_m;
        elseif strcmp(MemberName, 'd_modusmtx_p')
            calcDualModes()
            ret = d_modusmtx_p;
        elseif strcmp(MemberName, 'd_modusmtx_m')
            calcDualModes()
            ret = d_modusmtx_m; 
        elseif strcmp(MemberName, 'M')
            ret = M;
        elseif strcmp(MemberName, 'k_p')
            ret = k_p;     
        elseif strcmp(MemberName, 'k_m')
            ret = k_m;                
        elseif strcmp(MemberName, 'epsilon')
            ret = params.epsilon;
        elseif strcmp(MemberName, 'vargamma')
            ret = params.vargamma;            
        elseif strcmp(MemberName, 'params')
            ret = params;
        elseif strcmp(MemberName, 'param')
            ret = param;
        elseif strcmp(MemberName, 'Lead_Orientation')
            ret = Lead_Orientation;
        elseif strcmp(MemberName, 'Hanyadik_Lead')
            ret = Hanyadik_Lead;                    
        else
            disp(['No member to read with name ',MemberName]);
        end
    end
%% MemberClear
%   Clears the value of an element in the object.
    function MemberClear(MemberName)
        if strcmp(MemberName, 'H0')
            H0 = [];
        elseif strcmp(MemberName, 'H1')
            H1 = [];
        elseif strcmp(MemberName, 'H1adj')
            H1adj = [];
        elseif strcmp(MemberName, 'H00')
            H00 = [];
        elseif strcmp(MemberName, 'fazis_mtx_H0')
            fazis_mtx_H0 = [];            
        elseif strcmp(MemberName, 'fazis_mtx_H1')            
            fazis_mtx_H1 = [];
        elseif strcmp(MemberName, 'kulso_szabfokok')
            kulso_szabfokok = [];
        elseif strcmp(MemberName, 'coordinates')
            coordinates = [];
        elseif strcmp(MemberName, 'Normamtx')
            Normamtx = [];
        elseif strcmp(MemberName, 'gsurf')
            gsurf = [];
        elseif strcmp(MemberName, 'ginf')
            ginf = [];
        elseif strcmp(MemberName, 'gfin')
            gfin = [];            
        elseif strcmp(MemberName, 'vcsop_p')
            vcsop_p = [];
        elseif strcmp(MemberName, 'vcsop_m')
            vcsop_m = []; 
        elseif strcmp(MemberName, 'modusmtx_p')
            modusmtx_p = [];
        elseif strcmp(MemberName, 'modusmtx_m')
            modusmtx_m = []; 
        elseif strcmp(MemberName, 'd_modusmtx_p')
            d_modusmtx_p = [];
        elseif strcmp(MemberName, 'd_modusmtx_m')
            d_modusmtx_m = [];     
        elseif strcmp(MemberName, 'Hamiltonian2Dec')
        else
            disp(['No member to clear with name ',MemberName]);
        end
        
    end

% displays output messages on the screen
    function display( message )
       if Opt.Silent
           return
       else
           disp( message )
       end
        
    end

% Input parser
    function inputparser()
        p = inputParser;
        p.addParamValue('Way2Hamiltonian', []);
        p.addParamValue('Hanyadik_Lead', []);
        p.addParamValue('Lead_Orientation', 1);
        p.addParamValue('usingDualModes', 1);
        p.addParamValue('Lead_End_Type', 'A');
        p.addParamValue('Atom_Type', 'A');
        p.parse(varargin{:});

        Lead_Orientation  = p.Results.Lead_Orientation;
        Hanyadik_Lead     = p.Results.Hanyadik_Lead;
        Way2Hamiltonian   = p.Results.Way2Hamiltonian;
        usingDualModes    = p.Results.usingDualModes;
		Lead_End_Type     = p.Results.Lead_End_Type;
		Atom_Type    	  = p.Results.Atom_Type;
        
        
        if usingDualModes
            GinfSzamolo = @GinfSzamoloWithDualModes;
        else
            GinfSzamolo = @GinfSzamoloWithLeftModes;
        end
        
        setM();
        
        if isempty( Hanyadik_Lead )
            params        = param.scatter;  %Lead parameters
        else
            params        = param.Leads{Hanyadik_Lead};  %Lead parameters
        end
        
        
    end

%------------------------------------------------
    function setM()
        if isempty( Hanyadik_Lead )
            M             = param.scatter.shape.width;
        else
            M             = param.Leads{Hanyadik_Lead}.M;
        end        
    end


% End of Nested functions
%------------------------------------------------------------------
end % Global End
