%%    Favago Transport Programming Interface - OverlapingIntegrals
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    This function parses the input file containing input parameters
%%
function [ Opt, param ] = parseInput( filename )


%filename = [pwd,  '/Basic_Input_zigzag_leads.xml'];
%filename = [pwd,  '/Basic_Input.xml'];
Opt      = struct();
param    = struct();

parameter_names = { 'Hscatter', 'Hcoupling', 'Hleads', 'Decimation', 'Decimation_block', 'Decimate_only_Dyson', 'Potential', 'magnetic_field', 'my_own_potential', 'Lattice_Type', ...
    'Simple_Green_Function', 'Linear_Regression_in_B', 'Just_Create_Hamiltonians', 'saveWithMagneticFiled', 'BdG', 'Silent', 'type_of_scanning' };
hamiltonian_parameters = { 'epsilon', 'vargamma', 'SOintrinsic', 'deltaAB', 'SO_rashba_intrinsic', 'Overlap_in_Scatter', 'pair_potential' };

setDefaults()
parseXML( filename )

if Opt.Lattice_Type == 0
    Opt.Lattice_Type = 'S';
elseif Opt.Lattice_Type == 1
    Opt.Lattice_Type = 'H';
elseif Opt.Lattice_Type == 2
    Opt.Lattice_Type = 'Silicene';   
else
    %error('Wrong type of lattice');
end

    %% setDefaults
    % This sets default values to the parameter structures
    function setDefaults()
       
        Opt.Hscatter  = 0;
        Opt.Hcoupling = 0;
        Opt.Hleads    = 0;
        Opt.Decimation = 2;
        Opt.Decimation_block = 50;
        Opt.NofLeads = 0;
        Opt.Potential = 0;
        Opt.magnetic_field = 0;
        Opt.my_own_potential = 0;
        Opt.Lattice_Type = 0;
        Opt.Simple_Green_Function = 0;
        Opt.Linear_Regression_in_B = 0;
        Opt.Just_Create_Hamiltonians = 0;
        Opt.saveWithMagneticFiled = 0;
        Opt.BdG       = 0;
        Opt.Silent    = 0;
        Opt.type_of_scanning = 0;
                
        param.scatter.epsilon  = 0;
        param.scatter.vargamma = 0;
        param.scatter.shape = [];
        param.scatter.Overlap_in_Scatter = 0;
        
        param.Leads = cell(0);
        
    end
    
    %% parseXML
    % This opens the XML file and fills up the parameter structures
    function parseXML(filename)
        
        tree = xmlread(filename); 
        traverseNodes( tree );
        
    end


    %% traverseNodes
    % This parses over the attributes of the tree node
    function traverseNodes( theNode )
        % Recurse over node children.
        
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end


        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'parameters' )
                traverseNodes( theChild )
            elseif strcmp(char(theChild.getNodeName), 'computing_parameters' )
                setComputingParameters( theChild )
            elseif strcmp(char(theChild.getNodeName), 'Scatter_region' )
                setScatterParameters( theChild )
            elseif strcmp(char(theChild.getNodeName), 'Lead_parameters' )
                setLeadParameters( theChild )
            end
            
           
            
            %children(count) = makeStructFromNode(theChild);
        end
    end

    %% setComputingParameters
    % This sets computing parameters in the structures
    function setComputingParameters( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if ismember(char(theChild.getNodeName), parameter_names )
                attributes = parseAttributes(theChild);
                setOptValue( attributes, char(theChild.getNodeName) )
                
            elseif ismember(char(theChild.getNodeName), hamiltonian_parameters )
                attributes = parseAttributes(theChild);
                setParamValue( attributes, char(theChild.getNodeName) )
            end
            
            %traverseNodes( theChild )
            
            %children(count) = makeStructFromNode(theChild);
        end
        
    end


    %% setScatterParameter
    % This sets parameters for the scattering region
    function setScatterParameters( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if ismember(char(theChild.getNodeName), parameter_names )
                attributes = parseAttributes(theChild);
                setOptValue( attributes, char(theChild.getNodeName) )
                
            elseif ismember(char(theChild.getNodeName), hamiltonian_parameters )
                attributes = parseAttributes(theChild);
                setParamValue( attributes, char(theChild.getNodeName) )
                
            elseif strcmp(char(theChild.getNodeName), 'shape' )
                createShape( theChild )
            end
            
            %traverseNodes( theChild )
            
            %children(count) = makeStructFromNode(theChild);
        end
        
    end

    %% createShap
    % This creates shape for scattering region
    function createShape( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        shape = [];
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'width' )
                attributes = parseAttributes( theChild );
                shape.width = str2double( attributes.value );
            elseif strcmp(char(theChild.getNodeName), 'height' )
                attributes = parseAttributes( theChild );
                shape.height = str2double( attributes.value );
            elseif strcmp(char(theChild.getNodeName), 'Lead_End_Type' )
                attributes = parseAttributes( theChild );
                shape.Lead_End_Type = attributes.value;
             elseif strcmp(char(theChild.getNodeName), 'Atom_Type' )
                attributes = parseAttributes( theChild );
                shape.Atom_Type = attributes.value;               
            end  
        end
       
        param.scatter.shape = shape;
    end


    %% setLeadParameters
    % This sets parameters for Leads
    function setLeadParameters( theNode )
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'lead' )
                createLeadNode( theChild );
                
            end
            
            %traverseNodes( theChild )
            
            %children(count) = makeStructFromNode(theChild);
        end
        
    end

    %% createLeadNode
    % This fills up data from one lead node
    function createLeadNode( theNode )
        lead_attributes = parseAttributes( theNode );
        num = str2double(lead_attributes.num);
        
        try 
            params = param.Leads{num}; 
        catch
            params = struct('Lead_Orientation', [], 'M', [], 'Overlap_in_Leads', 0); 
        end
            
        if theNode.hasChildNodes
           childNodes = theNode.getChildNodes;
           numChildNodes = childNodes.getLength;
        else
            return
        end
        
        
        
        for count = 1:numChildNodes
            theChild = childNodes.item(count-1);
            
            if strcmp(char(theChild.getNodeName), 'epsilon' )
                attributes = parseAttributes( theChild );
                params.epsilon = str2double(attributes.value);
            elseif strcmp(char(theChild.getNodeName), 'vargamma' )
                attributes = parseAttributes( theChild );
                params.vargamma = str2double(attributes.value);
            elseif strcmp(char(theChild.getNodeName), 'SOintrinsic' )
                attributes = parseAttributes( theChild );
                params.SOintrinsic = str2double(attributes.value); 
            elseif strcmp(char(theChild.getNodeName), 'deltaAB' )
                attributes = parseAttributes( theChild );
                params.deltaAB = str2double(attributes.value);
            elseif strcmp(char(theChild.getNodeName), 'SO_rashba_intrinsic' )
                attributes = parseAttributes( theChild );
                params.SO_rashba_intrinsic = str2double(attributes.value); 
            elseif strcmp(char(theChild.getNodeName), 'pair_potential' )
                attributes = parseAttributes( theChild );
                params.pair_potential = str2double(attributes.abs)*exp(1i*str2double(attributes.phase));
             elseif strcmp(char(theChild.getNodeName), 'Overlap_in_Leads' )
                attributes = parseAttributes( theChild );
                params.Overlap_in_Leads = str2double(attributes.value);  
             elseif strcmp(char(theChild.getNodeName), 'orientation' )
                 attributes = parseAttributes( theChild );
                 params.Lead_Orientation = str2double(attributes.value);
             elseif strcmp(char(theChild.getNodeName), 'M' )
                 attributes = parseAttributes( theChild );
                 params.M = str2double(attributes.value);
            end
            
            
        end
        
        
        param.Leads{num}      = params;
        
    end


    %% setOptValue
    % This sets a value for a field in the Opt structure 
    function setOptValue( attributes, attribname )
       
        if ( isfield( attributes, 'value') )
           Opt.( attribname ) = str2double( attributes.('value') );
        else
            disp(['bad attribute: ', attribname ])
        end
        
    end

    %% setParamValue
    % This sets a value for a field in the Opt structure 
    function setParamValue( attributes, attribname )
        
        if ( isfield( attributes, 'value') )
           param.scatter.( attribname ) = str2double( attributes.('value') );
        elseif ( isfield( attributes, 'abs') ) && ( isfield( attributes, 'phase') )
           param.scatter.( attribname ) = str2double(attributes.abs)*exp(1i*str2double(attributes.phase));
        else
            disp(['bad attribute: ', attribname ])
        end
        
    end



    %% parseAttributes
    % This function parses the attributes of the XML node
    % ----- Local function PARSEATTRIBUTES -----
    function attributes = parseAttributes(theNode)
    % Create attributes structure.

    attributes = [];
    if theNode.hasAttributes
        theAttributes = theNode.getAttributes;
        numAttributes = theAttributes.getLength;
        attributes = struct();
    else
        return
    end

    for count = 1:numAttributes
        attrib = theAttributes.item(count-1);
        attributes.( char(attrib.getName) ) = char(attrib.getValue);
    end
end



end





